from zope.interface import implements, directlyProvides, providedBy
from zope.component import getMultiAdapter, queryUtility

from Acquisition import aq_base, aq_inner
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils
from Products.CMFPlone.browser.interfaces import INavigationPortlet, INavtreeStrategy
from Products.CMFPlone.browser.interfaces import INavigationQueryBuilder
from Products.CMFPlone.browser.interfaces import INavigationRoot
from Products.CMFPlone.browser.interfaces import IDefaultPage
from Products.CMFPlone.browser.portlets.navigation import NavigationPortlet
from Products.CMFPlone.browser.navtree import buildFolderTree
from Products.CMFPlone.browser.navtree import getNavigationRoot
from Products.CMFPlone.browser.navtree import NavtreeQueryBuilder, SitemapQueryBuilder
from Products.CMFPlone.browser.navtree import DefaultNavtreeStrategy, INavigationQueryBuilder
from Products.CMFPlone.browser.navigation import DefaultPage
from Products.CMFPlone.interfaces.Translatable import ITranslatable
from Products.NavigationManager.browser.interfaces import IMenu
from Products.NavigationManager.browser.interfaces import INavigationManagerRequest
from Products.NavigationManager.browser.interfaces import INavigationManagerTree
from Products.NavigationManager.browser.buildtopictree import buildTopicTree

def getApplicationRoot(obj):
    portal_url = getToolByName(obj, 'portal_url')	    
    portal = portal_url.getPortalObject()

    while not INavigationRoot.providedBy(obj) and aq_base(obj) is not aq_base(portal):
        obj = utils.parent(obj)

    return obj

def getMenu(context):
    menuid = getattr(context, 'navigationmanager_menuid', None)
    submenu = getattr(context, 'navigationmanager_submenu', None)
    if menuid is None or submenu == 'plone navigation':
        return None
    
    catalog = getToolByName(context, 'portal_catalog')
    query = { 'portal_type' : 'NavigationItem' }
    if menuid is not None:
        query['id'] = menuid 
    else:
        query['getUrl'] = context.absolute_url()
    if ITranslatable.isImplementedBy(context) and context.Language() is not None \
            and len(context.Language()) > 0:
        query['Language'] = context.Language()
    menuItems = catalog.searchResults( query )
    
    if len(menuItems) > 0:
        navmanager = getToolByName(context, 'portal_navigationmanager')
        obj = menuItems[0].getObject()
        prev = obj
        while aq_base(utils.parent(obj)) is not aq_base(navmanager):
            prev = obj
            obj = utils.parent(obj)
        return prev
    return None

def getCurrentItem(tree):
    for child in tree['children']:
        if child['currentParent']:
            return getCurrentItem(child)
        elif child['currentItem']:
            return child, tree

def removeEmptyFolders(tree):
    # remove all empty folders on the first level in the tree
    new_children = []
    for child in tree['children']:
        is_empty = getattr(child['item'], 'is_empty', False)
        if child['currentItem'] or child['currentParent'] or not is_empty:
            new_children.append(child)
    tree['children'] = new_children

    # recursively remove all empty folders lower down in the hierarchy
    for child in tree['children']:
        removeEmptyFolders(child)

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

class ListAllNode:
    """ This is an object that pretends to be a brain. """

    Title = 'List all'
    portal_type = 'Document'
    review_state = 'published'
    getUrl = 'atct_topic_full_view'

    def getPath(self):
        return 'dd'

    def getURL(self):
        return 'atct_topic_full_view'

    def getRemoteUrl(self):
        return None

    def Creator(self):
        return ''

    def Description(self):
        return ''

class NavigationManagerTree(utils.BrowserView):
    """ Navigation Tree which combines Navigation Manager menu and plone
        default. """
    
    implements(INavigationManagerTree)

    def navigationTreeRootPath(self):
        context = utils.context(self)
        return '/'.join(getApplicationRoot(context).getPhysicalPath())

    def navigationTree(self):
        mContext = context = utils.context(self)
        menu = getMenu(context)
        strategy = None
        queryBuilder = None
        isAnon = getToolByName(context, 'portal_membership').isAnonymousUser()
        
        if menu and isAnon:
            # we only use NavigationManager if we are anonymous
            menuTree = getMultiAdapter((context, self.request), name="eea_menu")
            mContext = menu
            queryBuilder = NavtreeManagerQueryBuilder(mContext)

        if queryBuilder is None:
            queryBuilder = INavigationQueryBuilder(mContext)

        query = queryBuilder()

        root = getApplicationRoot(context)
        if ITranslatable.isImplementedBy(root):
            query['Language'] = root.getLanguage()

        # Logged in users should get all objects in the navtree,
        # independent of the workflow state.
        if not isAnon:
            if query.has_key('review_state'):
                del query['review_state']

        query['is_default_page'] = {'query' : (True, False),
                                    'operator': 'or' }


        strategy = getMultiAdapter((mContext, self), INavtreeStrategy)
        reqInterfaces = providedBy(context.REQUEST)
        directlyProvides(context.REQUEST, reqInterfaces + INavigationManagerRequest)
        tree = buildFolderTree(mContext, obj=mContext, query=query, strategy=strategy)
        props = getToolByName(context, 'portal_properties')
        navtree_props = getattr(props, 'navtree_properties', None)
        if navtree_props is not None:
            topic_enabled = navtree_props.getProperty('topicListingInNavtree', False)
        else:
            topic_enabled = False

        portal_factory = getToolByName(mContext, 'portal_factory')
        temporary = portal_factory.isTemporary(mContext)

        # if context is a Smart Folder, then the navtree should display
        # the result of the smart folder query instead of folder contents

        if mContext.portal_type.endswith('Topic') and topic_enabled and \
                not temporary:
            topictree = buildTopicTree(mContext, obj=mContext, strategy=strategy)

            current, parent = getCurrentItem(tree)
            topiccurrent, topicparent = getCurrentItem(topictree)
            if isAnon:
                parent['children'] = topiccurrent['children']
            else:
                parent['children'] = [current]
                current['children'] = topiccurrent['children']

            item = ListAllNode()
            listAllNode = {'item'      : item,
                           'depth'         : 0,  # irrelevant
                           'currentItem'   : False,
                           'currentParent' : False,
                           'children': [] }

            if strategy is not None:
                listAllNode = strategy.decoratorFactory(listAllNode)

            topiccurrent['children'].append(listAllNode)

        directlyProvides(context.REQUEST, reqInterfaces)
        for m in tree['children']:
            if menu:
                m['getURL'] = m['item']['getUrl']
            selectedMenuId = getattr(context, 'navigationmanager_menuid', None)
            currentContext = m['getURL'] == context.absolute_url()
            currentMenuItem = selectedMenuId and m['item']['getId'] == selectedMenuId
            m['currentItem'] = currentContext or currentMenuItem or m['currentItem'] and not m['currentParent']
            if m['currentParent']:
                for child in m['children']:
                    if child['defaultPage']:
                        idx = m['children'].index(child)
                        m['children'].pop(idx)
                        m['children'].insert(0, child)
                        break
            if m.get('defaultPage', False):
                idx = tree['children'].index(m)
                tree['children'].pop(idx)
                tree['children'].insert(0, m)

        # anonymous users don't want to see empty folders
        if isAnon and not menu:
            removeEmptyFolders(tree)
        return tree


class NavigationManagerPortlet(NavigationPortlet):
    """ EEA website navigation portlet fetches menu from navigation manager. """
    
    implements(INavigationPortlet)

    def __init__(self, context, request):
        NavigationPortlet.__init__(self, context, request)
        mship = getToolByName(context, 'portal_membership')
        isAnonymous = mship.isAnonymousUser()
        if isAnonymous:
            root = getMenu(context)
            if root is not None:
                self._root = [ root ]
            
    def title(self):
        return self.navigationRoot().Title()
    
    def navigationRoot(self):
        """ Override """
        if not utils.base_hasattr(self, '_root'):
            self._root = [ NavigationPortlet.navigationRoot(self) ]
        return self._root[0]

class NavtreeManagerStrategy(DefaultNavtreeStrategy):
    """ The navtree strategy used for the default navigation portlet and
        respects NavigationManager submenu as root.  """
    
    implements(INavtreeStrategy)

    def __init__(self, context, view=None):
        DefaultNavtreeStrategy.__init__(self, context, view)
        self.rootPath = '/'.join(context.getPhysicalPath())
        self.showAllParents = False

class NavtreeSectionStrategy(DefaultNavtreeStrategy):
    """ The navtree strategy that provides navigation section info for each node. """
        
    def decoratorFactory(self, node):
        newNode = DefaultNavtreeStrategy.decoratorFactory(self, node)
        item = node['item']
        newNode['navSection'] = getattr(item, 'navSection', None) or 'default'
        newNode['defaultPage'] = getattr(item, 'is_default_page', False)
        return newNode
    
class TopicNavtreeStrategy(NavtreeSectionStrategy):
    """ The navtree strategy for topics. """
 
    def __init__(self, context, view=None):
        NavtreeSectionStrategy.__init__(self, context, view)
        portal_url = getToolByName(context, 'portal_url')
        portal = portal_url.getPortalObject()
        #self.rootPath = '/'.join(context.getPhysicalPath())
        #self.rootPath = '/'.join(portal.getPhysicalPath())
        if view is not None:
            self.rootPath = view.navigationTreeRootPath()
        else:
            self.rootPath = getNavigationRoot(context)
        self.showAllParents = True


    def decoratorFactory(self, node):
        newNode = NavtreeSectionStrategy.decoratorFactory(self, node)
        context = utils.context(self)
        item = node['item']
        #newNode['path'] = context.absolute_url() + '/' + item.getId
        newNode['defaultPage'] = getattr(item, 'is_default_page', False)
        return newNode

class NavtreeManagerQueryBuilder:
    """Build a navtree query based on the settings in navtree_properties
    """
    implements(INavigationQueryBuilder)

    def __init__(self, context):
        portal_properties = getToolByName(context, 'portal_properties')
        portal_url = getToolByName(context, 'portal_url')
        navtree_properties = getattr(portal_properties, 'navtree_properties')

        query = {}
        rootPath = currentPath = '/'.join(context.getPhysicalPath())

        # If we are above the navigation root, a navtree query would return
        # nothing (since we explicitly start from the root always). Hence,
        # use a regular depth-1 query in this case.

        if not currentPath.startswith(rootPath):
            query['path'] = {'query' : rootPath, 'depth' : 2}
        else:
            query['path'] = {'query' : currentPath, 'navtree' : 2}

        # portal/portal_navigationmanager/site/menu = 4
        query['path']['navtree_start'] = 4

        # XXX: It'd make sense to use 'depth' for bottomLevel, but it doesn't
        # seem to work with EPI.

        # Only list the applicable types
        query['portal_type'] = utils.typesToList(context)

        # Apply the desired sort
        sortAttribute = navtree_properties.getProperty('sortAttribute', None)
        if sortAttribute is not None:
            query['sort_on'] = sortAttribute
            sortOrder = navtree_properties.getProperty('sortOrder', None)
            if sortOrder is not None:
                query['sort_order'] = sortOrder

        # Filter on workflow states, if enabled
        if navtree_properties.getProperty('enable_wf_state_filtering', False):
            query['review_state'] = navtree_properties.getProperty('wf_states_to_show', ())

        self.query = query

    def __call__(self):
        return self.query


class NavtreeManagerQueryBuilderForSections(NavtreeQueryBuilder):

    def __init__(self, context):
        NavtreeQueryBuilder.__init__(self, context)
        if self.query.get('depth', None) is not None:
            self.query['depth'] = 2
        else:
            self.query['navtree'] = 2            


class DefaultPageIsNormalPage(DefaultPage):
    """ Default page is only hidden if it's excluded from navigation. """
    implements(IDefaultPage)

    def isDefaultPage(self, obj, context_=None):
        return DefaultPage.isDefaultPage(self, obj, context_) and obj.exclude_from_nav() or False
    
